import argparse

from rkale.operations import handle_copy, handle_sync
from rkale.project_operations import handle_pcopy, handle_psync

OPERATIONS = {
    "copy": handle_copy,
    "sync": handle_sync,
    "pcopy": handle_pcopy,
    "psync": handle_psync,
}


def main():
    parser = argparse.ArgumentParser()

    base_parser = argparse.ArgumentParser(add_help=False)
    base_parser.add_argument(
        "-f",
        "--force",
        action="store_true",
        dest="force",
        help="Run operation without checks",
    )

    subparsers = parser.add_subparsers(title="operation", dest="operation")
    global_parser = subparsers.add_parser(
        "copy", aliases=["sync"], parents=[base_parser]
    )
    global_parser.add_argument("source")
    global_parser.add_argument("destination")
    global_parser.set_defaults(
        func=lambda args: OPERATIONS[args.operation](
            [(args.source, args.destination)], force=args.force
        )
    )

    project_parser = subparsers.add_parser(
        "pcopy", aliases=["psync"], parents=[base_parser]
    )
    project_parser.add_argument(
        "-u",
        "--upstream",
        action="store_true",
        dest="upstream",
        help="Apply changes upstream",
    )
    project_parser.set_defaults(
        func=lambda args: OPERATIONS[args.operation](
            force=args.force, upstream=args.upstream
        )
    )

    args = parser.parse_args()
    try:
        func = args.func
        func(args)
    except AttributeError:
        parser.error("too few arguments")
    except Exception as e:
        print(str(e).rstrip("\n"))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rkale']

package_data = \
{'': ['*']}

install_requires = \
['toml>=0.10.2,<0.11.0', 'tqdm>=4.60.0,<5.0.0']

entry_points = \
{'console_scripts': ['rkale = rkale.rkale:main']}

setup_kwargs = {
    'name': 'rkale',
    'version': '0.2.8',
    'description': 'Rclone wrapper to manage multiple datasets in a project',
    'long_description': '# rkale\n\n## Install\n\nInstall lantern in your project using poetry:\n\n```bash\npoetry add rkale\n```\n\nUse pip if you want a global installation:\n\n```bash\npip install rkale\n```\n\n## Configuration\n\n### Global\n\n`~/.config/rkale/rkale.conf`:\n```toml\n[data]\nroot = "path to data folder where datasets are stored"\n\n[aliases]\nwasabi = "optional alias for remote in rclone.conf"\n\n[rclone] # global flags for rclone\nflags = ["--transfers 32", "--checkers 32"]\n```\n\nIf alases are empty the remote name from the project config is used in the rclone lookup.\n\n### Project\nConfigure project datasets in the pyproject.toml file:\n\n`<project path>/pyproject.toml`:\n```toml\n[[tool.rkale.dataset]]\nname = "dataset_1"\nremote = "remote_1"\n\n[[tool.rkale.dataset]]\nname = "dataset_2"\nremote = "remote_2"\n```\n\nThe remote specified for the dataset must match a remote in the `rclone.conf` or an alias in the global rkale configuration.\n\n## Usage\n\n### Python interface\n\n```python\nfrom rkale.config import dataset_paths\n\nDATASET_PATH = dataset_paths()["dataset_1"]\n```\n\n### Syncing datasets\n\nSyncs the local datasets to be identical to the remote\n\n```bash\nrkale psync\n```\n\nSyncs the remote datasets to be identical to the local\n\n```bash\nrkale psync --upstream\n```\n\nSame as rclone sync but checks differences first and asks for confirmation\n\n```bash\nrkale sync <source> <destination>\n```\n',
    'author': 'Joar Gruneau',
    'author_email': 'joar@aiwizo.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Aiwizo/rkale',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import os
from functools import lru_cache
from pathlib import Path

import toml
from rkale.exceptions import ConfigError, DataRootError, DatasetError


@lru_cache()
def global_configuration():
    rkale_config_path = Path.home() / ".config/rkale/rkale.conf"
    if rkale_config_path.exists():
        return load_configuration(rkale_config_path)
    else:
        return dict(
            data=dict(
                root=str(Path.home() / "rkale-data"),
            )
        )


@lru_cache()
def project_configuration(working_dir=None):
    if working_dir is None:
        working_dir = os.getcwd()
    config_path = find("pyproject.toml", Path(working_dir))
    config = load_configuration(config_path)
    if "tool" in config and "rkale" in config["tool"]:
        return config["tool"]["rkale"]
    raise ConfigError(f"No rkale section found in {config_path}")


def rclone_flags():
    config = global_configuration()
    if "rclone" in config and "flags" in config["rclone"]:
        return config["rclone"]["flags"]
    return []


def datasets(working_dir=None):
    if working_dir is None:
        working_dir = os.getcwd()
    config = project_configuration(working_dir=working_dir)
    if "dataset" in config:
        return config["dataset"]
    raise DatasetError("No dataset defined in rkale config")


def dataset_paths(working_dir=None):
    if working_dir is None:
        working_dir = os.getcwd()
    data_root = Path(global_configuration()["data"]["root"])
    return {
        dataset["name"]: path(data_root, dataset["name"])
        for dataset in datasets(working_dir=working_dir)
    }


def path(root, name):
    resolved_path = Path(root.expanduser() / name).resolve()
    try:
        resolved_path.relative_to(root.expanduser().resolve())
        if resolved_path != root:
            return resolved_path
    except ValueError:
        pass
    raise DataRootError(f"Path {resolved_path} is outside data root {root}")


def find(name, root):
    while root != Path("/"):
        if (root / name).exists():
            return root / name
        root = root.parent
    raise FileNotFoundError(f"Could not find file {name} in path {root} or any of it's parent directories")


def load_configuration(path):
    with open(path) as f:
        value = toml.load(f)
        return value
